<?php

/*
	ezxphoneassist.cpp
	It's a utility for enhance calling on ezx phone platform.
	a configure file is needed,like below
//	------------------------------------------------------------
	#ezxphoneassist.cfg
	#comment line...
	[setting]
	connectedshake=true
	dropedshake=true
	#firewalltype can be black,white or none
	firewalltype=black
	[callblacklist]
	#139*
	#*12345
	#??273?2?8
	[callwhitelist]
	#010???8*
//	------------------------------------------------------------
	Copyright (C) 2006 by psysun(pesky@yeah.net)
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version. 
//	------------------------------------------------------------
	ezxphoneassist GUI version for ROKR E2
	Created by Jeffri Hong
	Originally GUI version created by DDSH
*/

define("MYPATH", dirname($_SERVER['PHP_SELF']));
define("TITLE", "Black&White List");
define("ICON", MYPATH."/ezxphoneassistB.png");
$config_files = "/mmc/mmca1/blackno.txt";
require("show.php");

$run_as = `whoami`; // lets check if the apps run with root or not
if ($run_as != "root\n")
{
	// oh no, just stop here... :)
	$error = new showNotify("Error", "Not run as root");
	$error->exec();
	die("Not run as root.\n");
}

// Regex and wildcard can be very confusing and cause error everywhere, so here is a little function to fix it
function regex_wildcard_fix($text)
{
	$text = str_replace("*", "\*", $text);
	$text = str_replace("?", "[?]", $text); // Im not sure why, but \? never work for me on ereg, but using surrounded square brackets will work!
	return $text;
}

// Here is the configuration class for managing ezxphoneassist configuration
class configuration
{
	public function __construct($cfg)
	{
		// We pass configuration files on constructor and constructor will open and read the files
		if ($opn = fopen($cfg, "r"))
		{
			$this->file = $cfg;
			$s = 0;
			while (! feof($opn))
			{
				$getline = fgets($opn);
				if (ereg("firewalltype=([a-z]*)", $getline, $get))
				{
					// The type current running, could be white or black
					$this->type = $get[1];
					unset($get);
				}
				if (ereg("\[(.*)\]", $getline, $get))
				{
					// This will define the current section on the configuration
					$section = $get[1];
					$this->section[$section][] = $section; // the first value of this array will be the section itself
					unset($get);
				}
				elseif (isset($section))
				{
					$this->section[$section][] = $getline; // this will add every line of after the section to the array, it will deal with white/black list numbers but no with other configuration, well, we dont need to care about others :)
				}
			}
			fclose($opn);
		}
	}
	public function get_type()
	{
		return $this->type; // return the current run type
	}
	public function get_black_list()
	{
		$this->reset();
		$out = $this->section['callblacklist'];
		unset($out[0]);
		return $out; // return the array set of blacklist number
	}
	public function get_white_list()
	{
		$this->reset();
		$out = $this->section['callwhitelist'];
		unset($out[0]);
		return $out; // return the array set of whitelist number
	}
	public function add($section, $value)
	{
		// A handy  method for adding number, this will threat whitespace for divider of each number, so adding multiple number is possible here
		$recent_array = $this->section[$section];
		unset($recent_array[0]);
		$recent = implode("", $recent_array);
		
		$value = ereg_replace(" +$", "", $value); // removing the whitespace on end of string, if any
		$value = ereg_replace(" +", "\n", $value); // change whitespace(s) to a newline
		if (ereg("\n", $value))
		{
			$new_array = explode("\n", $value);
			foreach ($new_array as $num)
			{
				if (ereg("^".regex_wildcard_fix($num)."\n$", $recent))
				{
					$value = str_replace("$num\n", "", $value);
				}
			}
		}
		else
		{
			if (ereg("^".regex_wildcard_fix($value)."\n$", $recent))
			{
				return false;
			}
		}
		$new_value = $recent.$value."\n";
		$this->edit_config($section, $recent, $new_value);
		return true;
	}
	public function edit($section, $old_val, $new_val)
	{
		$recent_array = $this->section[$section];
		unset($recent_array[0]);
		$recent = implode("", $recent_array);
		
		$new_val = ereg_replace(" +$", "", $new_val); // removing the whitespace on end of string, if any
		$new_val = ereg_replace(" +", "\n", $new_val); // change whitespace(s) to a newline
		if (ereg("\n", $new_val))
		{
			$new_array = explode("\n", $new_val);
			foreach ($new_array as $num)
			{
				if (ereg("^".regex_wildcard_fix($num)."\n$", $recent))
				{
					if ($num != $old_val)
						$new_val = str_replace("$num\n", "", $new_val);
				}
			}
		}
		else
		{
			if (ereg("^".regex_wildcard_fix($new_val)."\n$", $recent))
			{
				return false;
			}
		}
		$new_value = str_replace("$old_val", "$new_val\n", $recent);
		$this->edit_config($section, $recent, $new_value);
		return true;
	}
	public function remove($section, $value_arr)
	{
		// Another handy method for remove number, the $value_arr is an array with all numbers to remove
		$recent_array = $this->section[$section];
		unset($recent_array[0]);
		$recent = implode("", $recent_array);
		
		$new_value = $recent;
		foreach ($value_arr as $select)
		{
			$new_value = str_replace("$select", "", $new_value);
		}
		$this->edit_config($section, $recent, $new_value);
		return true;
	}
	public function change_type()
	{
		// And this is the method to change the type, run this will change it to black/white, depend on the last configuration
		$this->reset();
		$recent_array = $this->section['setting'];
		unset($recent_array[0]);
		$recent = implode("", $recent_array);

		if ($this->type == "black")
			$value = "white";
		else
			$value = "black";

		$new_value = ereg_replace("(firewalltype=)[a-z]*", "\\1$value", $recent);
		$this->edit_config("setting", $recent, $new_value);
		return true;
	}
	public function reset()
	{
		unset($this->section);
		$this->__construct($this->file);
	}
	protected function edit_config($section, $old_val, $new_val)
	{
		// this is the method for editing black/whitelist number
		if ($opn = fopen($this->file, "r"))
		{
			while (! feof($opn))
			{
				$read = fread($opn, 4096);
			}
			fclose($opn);
		}
		else
		{
			echo "Error on opening files.\n";
		}
		$edit = str_replace("[$section]\n$old_val", "[$section]\n$new_val", $read);
		if ($opn = fopen($this->file, "w"))
		{
			fwrite($opn, $edit);
			fclose($opn);
		}
		else
		{
			echo "Error on writing files.\n";
		}
	}
}

$main = 1;
// Set all instance first
$interface['main'] = new showList(TITLE, null, null, "Select select Help help About about", "Exit");
$interface['black'] = new showList(TITLE, "Blacklist numbers:", null, "Edit edit Add add Delete delete", "Back");
$interface['white'] = new showList(TITLE, "Whitelist numbers:", null, "Edit edit Add add Delete delete", "Back");
$interface['add'] = new showEntry("Add Number", "Use whitespace to divide each number in the case you want to add multiple numbers, wildcard allowed");
$interface['edit'] = new showEntry("Edit Number", "Write the new number/wildcard");
$interface['delete'] = new showMultiSelect("Delete Number", "Choose the number you want to remove from list");
$interface['help'] = new showQ("Help", null, 2);
$interface['about'] = new showQ("About", null, 2);
$interface['confirm'] = new showQ("Confirmation", null, 1);
$interface['error'] = new showQ("Error", null, 2);

foreach ($interface as $index)
{
	$index->set_icon(ICON); // this will apply icon for all show* instance 
}

if (! file_exists("/ezxlocal/ezxphoneassist"))
{
	copy(MYPATH."/ezxphoneassist", "/ezxlocal/ezxphoneassist");
	chmod("/ezxlocal/ezxphoneassist", 777);
}

while ($main != 0)
{
	// check for the configs instance
	if (isset($configs))
		$configs->reset();
	else
		$configs = new configuration($config_files);
		

	// check for ezxphoneassist process
	$run_proc = `ps h -A -o %c|grep "^ezxphoneassist$"`;
	if (! empty($run_proc))
	{
		$status = "Stop"; // if active already
		$str_status = "Running";
	}
	else
	{
		$status = "Run"; // if not active
		$str_status = "Not running";
	}
	
	// check for running type
	switch ($configs->get_type())
	{
		case "black":
			$mode = "Change to White Mode";
			$str_mode = "Black mode";
			break;
		case "white":
			$mode = "Change to Black Mode";
			$str_mode = "White mode";
			break;
		default:
			$mode = "Change to Black Mode";
			$str_mode = "None";
	}
	
	$interface['main']->set_context("Status: $str_status\nMode: $str_mode");
	$interface['main']->set_list("'$status' 'Manage Blacklists' 'Manage Whitelists' '$mode'");
	$ret = $interface['main']->exec();
	$option = $ret[0];
	$main = $ret[1];
	
	if ($main == $option || $option == "select")
	{
		// no option selected or user clicked on "Select"
		switch ($main)
		{
			case 1:
			// run or stop here
				if ($status == "Run")
				{
					chmod("/ezxlocal/ezxphoneassist", 777);
					exec("/ezxlocal/ezxphoneassist > /dev/null &");
				}
				else
				{
					exec("kill -9 `pidof ezxphoneassist`");
				}
				break;
			case 2:
			// manage blacklists
				$manage_method = "get_black_list";
				$manage_mode = "black";
				$manage_section = "callblacklist";
				break;
			case 3:
			// manage whitelists
				$manage_method = "get_white_list";
				$manage_mode = "white";
				$manage_section = "callwhitelist";
				break;
			case 4:
			// change mode
				$configs->change_type();
				break;
		}
		
		if ($main == 2 || $main == 3)
		{
			$option2 = "";
			while ($option2 != "Cancel!")
			{
				// here we use printed text ($option2) instead of return code ($manage) for looping, the reason is because showList will always return 0 if there is no lists, so this will fix things
				$lists = $configs->$manage_method();
				$lists = "'".implode("' '", $lists)."'"; // listing the array to string
				$lists = str_replace("\n", "", $lists); // delete any newline characters
				$lists = str_replace("''", "", $lists); // delete empty string
				$interface[$manage_mode]->set_list($lists);
				$ret = $interface[$manage_mode]->exec();
				$option2 = $ret[0];
				$manage = $ret[1];
				
				switch ($option2)
				{
					case "add":
						$ret = $interface['add']->exec();
						if ($ret[1] == 1)
							$new_value = $ret[0];
						else
							break;
						if ($configs->add($manage_section, $new_value))
						{
							break;
						}
						else
						{
							$interface['error']->set_context("$new_value are already in the list");
							$interface['error']->exec();
							break;
						}
						
					case "delete":
						$number_list = $configs->$manage_method();
						$multi_lists = implode(" 0 ", $number_list)." 0"; // listing the array to string
						$multi_lists = str_replace("\n", "", $multi_lists); // delete any newline characters
						$multi_lists = ereg_replace("  0$", "", $multi_lists); // delete last 0 string with no pair (notice the two spaces in begining)
						$interface['delete']->set_list($multi_lists);
						$ret = $interface['delete']->exec();
						if ($ret[1] > 0)
						{
							if (isset($delete_array))
								unset($delete_array);
							foreach ($ret[0] as $select)
							{
								$select = ereg_replace("^0*", "", $select);
								$delete_array[] = $number_list[$select];
							}
							if ($configs->remove($manage_section, $delete_array))
							{
								break;
							}
							else
							{
								$interface['error']->set_context("An error occured while removing selected number.");
								$interface['error']->exec();
								break;
							}
							
						}
						break;
						
					case "edit":
					default:
						if ($manage == 0)
							break;
						$number_list = $configs->$manage_method();
						$current_number = ereg_replace("\n$", "", $number_list[$manage]); // remove the new line on end of string (fix bugs on showEntry)
						$interface['edit']->set_default($current_number); // set the current selected number as default
						$ret = $interface['edit']->exec();
						if ($ret[1] == 1)
							$new_value = $ret[0];
						else
							break;
						if ($configs->edit($manage_section, $number_list[$manage], $new_value))
						{
							break;
						}
						else
						{
							$interface['error']->set_context("Edit ".$number_list[$manage]." to $new_value failed.");
							$interface['error']->exec();
							break;
						}
						break;

				}
			}
		}
	}
	else
	{
		switch($option)
		{
			case "help":
				$help_text = "Black&White List is a GUI version of ezxphoneassist written by psysun (see About), so in this version, manage blacklist/whitelist number(s) is very easy. The usage is simple, just manage number(s), then select mode, and run. This also will append the command to run ezxphoneassist on autorun.txt.

If your inputed numbers can't work, you might need to use wildcard. So for example, you want to block call from 99999999, and this can't work for you, then you can use *99999999 instead. Or ?622199999999 can work too (6221 is your country and region code). This works for me, so I hope it will work for you too.";
				$interface['help']->set_context($help_text);
				$interface['help']->exec();
				break;
			case "about":
				$about_text = "Black&White List
Created by Jeffri Hong
August 16, 2008

Originally created by DDSH

ezxphoneassist
Copyright (C) 2006 by psysun(pesky@yeah.net)";
				$interface['about']->set_context($about_text);
				$interface['about']->exec();
				break;
		}
	}
}



?>
